import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

import utils

def plot_cv(data=None, labels=None, line=True, scatter=True, potential_column=0, current_column=1, xunits='V', yunits=r'$mA/cm^2$', export_name=None, export_type='png', figsize=(5,5), dpi=600):
	if not utils.check_dict(data):
		data = {'key': data}
	if export_name:
		fig, ax = plt.subplots(figsize=figsize, dpi=dpi)
	else:
		fig, ax = plt.subplots()
	for k, df in data.items():
		if 'i' in df.columns and 'v' in df.columns:
			x = df['v']
			y = df['i']
		else:
			x = df.iloc[:, potential_column]
			y = df.iloc[:, current_column]
		ax.plot(x, y, label=k)
	if len(data) > 1:
		ax.legend(loc='best')
	ax.set_xlabel('Potential [' + xunits + ']')
	ax.set_ylabel(r'Current Density [' + yunits + ']')
	if export_name:
		export_type = export_type.replace('.', '')
		if len(export_name.split('.')) == 1:
			export_name = export_name + '.' + export_type
		plt.savefig(export_name)
	return fig, ax

def plot_cp_raw(data=None, scatter=True, export_name=None, export_type='png', figsize=(5,5), dpi=600):
	if not utils.check_dict(data):
		data = {'key': data}
	if export_name:
		fig, ax1 = plt.subplots(figsize=figsize, dpi=dpi)
	else:
		fig, ax1 = plt.subplots()
	ax2 = ax1.twinx()
	color1 = 'tab:red'
	color2 = 'tab:blue'
	for k, df in data.items():
		x = df['t']
		y1 = df['i']
		y2 = df['v']
		ax1.scatter(x, y1, s=1, color=color1, label=k)
		ax2.scatter(x, y2, s=1, color=color2, label=k)
	if len(data) > 1:
		ax1.legend(loc='best')
	# color = 'tab:red'
	ax1.set_xlabel('Time [s]')
	ax1.set_ylabel('Current [mA]')
	ax1.tick_params(axis='y', labelcolor=color1)
	# color = 'tab:blue'
	ax2.set_ylabel('Potential [V]')
	ax2.tick_params(axis='y', labelcolor=color2)
	if export_name:
		export_type = export_type.replace('.', '')
		if len(export_name.split('.')) == 1:
			export_name = export_name + '.' + export_type
		plt.savefig(export_name)
	return fig, ax1, ax2

def polcurve(data=None, labels=None, line=True, scatter=True, potential_column=0, current_column=1, xunits='V', yunits=r'$mA/cm^2$', export_name=None, export_type='png', figsize=(5,5), dpi=600):
	if not utils.check_dict(data):
		data = {'key': data}
	if export_name:
		fig, ax = plt.subplots(figsize=figsize, dpi=dpi)
	else:
		fig, ax = plt.subplots()
	for k, df in data.items():
		if 'i' in df.columns and 'v' in df.columns:
			y = df['v']
			x = df['i']
		else:
			y = df.iloc[:, potential_column]
			x = df.iloc[:, current_column]
		if line and scatter:
			ax.plot(x, y, label=k, marker='.')
		elif line:
			ax.plot(x, y, label=k)
		else: 
			ax.scatter(x, y, label=k)
	if len(data) > 1:
		ax.legend(loc='best')
	ax.set_xlabel('Potential [' + xunits + ']')
	ax.set_ylabel(r'Current Density [' + yunits + ']')
	ax.set_title('Polarization Curves')
	if export_name:
		export_type = export_type.replace('.', '')
		if len(export_name.split('.')) == 1:
			export_name = export_name + '.' + export_type
		plt.savefig(export_name)
	return fig, ax

def plot_general(data=None, labels=None, line=True, scatter=True, xcolumn=0, ycolumn=1, xlabel='x', ylabel='y', export_name=None, export_type='png', figsize=(5,5), dpi=600):
	if not utils.check_dict(data):
		data = {'key': data}
	if export_name:
		fig, ax = plt.subplots(figsize=figsize, dpi=dpi)
	else:
		fig, ax = plt.subplots()
	for k, df in data.items():
		x = df.iloc[:, xcolumn]
		y = df.iloc[:, ycolumn]
		if line and scatter:
			ax.plot(x, y, label=k, marker='.')
		elif line:
			ax.plot(x, y, label=k)
		else: 
			ax.scatter(x, y, label=k)
	if len(data) > 1:
		ax.legend(loc='best')
	ax.set_xlabel(xlabel)
	ax.set_ylabel(ylabel)
	ax.set_title('Polarization Curves')
	if export_name:
		export_type = export_type.replace('.', '')
		if len(export_name.split('.')) == 1:
			export_name = export_name + '.' + export_type
		plt.savefig(export_name)
	return fig, ax



import numpy as np
import pandas as pd
import os
import re

import utils

dlm = utils.dlm_default

def load_data(filename=None, folder=None, pattern='', expttype='', delimiter=dlm, filetype=''):
	data = None
	if filename:
		data = utils.read_file(filename, delimiter)
	else:
		if folder:
			dirpath = os.path.realpath(folder)
		else:
			dirpath = os.getcwd()
		if expttype and not pattern:
			pattern = r'.*' + expttype + r'.*'
		files = utils.get_files(dirpath, pattern, filetype)
		data = dict()
		for f in files:
			path = os.path.join(dirpath, f)
			name, df = utils.read_file(path, delimiter)
			if df is not None:
				data[name] = df
		if len(data) == 1:
			data = list(data.values())[0]
	return data

def cv_raw(filename=None, folder=None, pattern='', delimiter=dlm):
	data = load_data(filename, folder, pattern, 'cv', delimiter)
	return data

def cp_raw(filename=None, folder=None, pattern='', delimiter=dlm):
	data = load_data(filename, folder, pattern, 'cp', delimiter)
	return data

def cv_process(data=None, area=5, current_column=1, filename=None, folder=None, pattern='', delimiter=dlm):
	if data is None:
		data = cv_raw(filename, folder, pattern, delimiter)
	if not utils.check_dict(data):
		data = {'key': data}
	newdata = dict()
	for k, df in data.items():
		newdf = df.copy()
		newdf.columns = utils.check_labels(newdf)
		if 'i' in newdf.columns:
			newdf['i'] = newdf['i'] / area
		else:
			newdf.iloc[:,current_column] = newdf.curr[:,current_column] / area
		newdata[k] = newdf
	if len(newdata) == 1:
		newdata = list(newdata.values())[0]
	return newdata

def cp_process(data=None, area=5, current_column=1, potential_column=2, aoi=True, filename=None, folder=None, pattern='', delimiter=dlm):
	if data is None:
		data = cp_raw(filename, folder, pattern, delimiter)
	if not utils.check_dict(data):
		data = {'key': data}
	newdata = dict()
	for k, df in data.items():
		newdf = df.copy()
		newdf.columns = utils.check_labels(newdf)
		# time = np.array(newdf['t'])
		if 'i' in newdf.columns and 'v' in newdf.columns:
			current = np.array(newdf['i'])
			potential = np.array(newdf['v'])
		else:
			current = np.array(newdf.iloc[:, current_column])
			potential = np.array(newdf[:, current_column])
		split_pts = findsteps(current, threshold=5)
		current_steps = split_and_filter(current, split_pts, min_length=100)
		potential_steps = split_and_filter(potential, split_pts, min_length=100)
		# time_steps = split_and_filter(potential, split_pts, min_length=100)
		current_avg = np.array([avg_last_pts(s) for s in current_steps])
		potential_avg = np.array([avg_last_pts(s) for s in potential_steps])
		if aoi:
			current_avg = avg_outside_in(current_avg)
			potential_avg = avg_outside_in(potential_avg)
		current_avg = current_avg / 5
		cp_processed = pd.DataFrame({'i':current_avg, 'v':potential_avg})
		newdata[k] = cp_processed
	if len(newdata) == 1:
		newdata = list(newdata.values())[0]
	return newdata

def findsteps(data, threshold=5):
	diffs = np.abs(np.diff(data))
	splits = np.where(diffs > threshold)[0] + 1
	return splits

def split_and_filter(data, split_pts, min_length=0):
	steps = np.split(data, split_pts)
	steps_tokeep = np.array([s for s in steps if len(s) > min_length])
	return steps_tokeep

def avg_last_pts(arr, numpts=300):
	avg = np.mean(arr[-numpts:])
	return avg

def avg_outside_in(arr):
	l = len(arr)
	avg = np.array([(arr[i]+arr[l-1-i])/2 for i in np.arange(l//2 + l%2)])
	return avg


import numpy as np
import pandas as pd
import os
import re
import logging

logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.INFO)
valid_types = ['csv', 'xls', 'xlsx', 'txt']
excel_types = ['xls', 'xlsx']
csv_types = ['csv', 'txt']
dlm_default = '\t'

label_dict = {'v':'v', 'ma':'i', 'a':'i', 's':'t', 'mv':'v',
				'v vs. sce':'v', 'mv vs. sce':'v', 'v vs. she':'v', 'mv vs. she':'v'}
default_labels = ['tintin', 'snowy', 'haddock', 'calculus', 'castafiore', 'thomson', 'thompson']

def check_type(filetype):
	filetype = filetype.lower().replace('.', '')
	if filetype not in valid_types:
		raise ValueError('Supported filetypes ' + ', '.join(valid_types))
	return filetype

def check_list(var):
	return (type(var) == list) or (type(var) == np.ndarray)

def check_dict(var):
	return type(var) == dict

def check_scalar(var):
	try:
		len(var)
		return False
	except:
		return True

def check_labels(data):
	cols = [c.lower() for c in data.columns]
	newcols = [];
	for c in cols:
		try:
			units = c.split('/')[1]
			if units in label_dict.keys():
				newcols.append(label_dict[units])
			else:
				newcols.append(c)
		except:
			newcols.append(c)
	return newcols

def get_files(path=None, pattern='', filetype=''):
	files = os.listdir(path)
	if pattern:
		files = [f for f in files if re.match(pattern, f)]
	if filetype:
		filetype = filetype.lower().replace('.', '')
		files = [f for f in files if re.match(r'.*\.'+filetype, f)]
	return files

def read_file(filename, dlm=dlm_default):
	data = None
	name = None
	try:
		name = os.path.basename(filename)
		name, filetype = name.split('.')
		filetype = check_type(filetype)
		if filetype in excel_types:
			data = pd.read_excel(filename)
		elif filetype in csv_types:
			if filetype == 'csv':
				data = pd.read_csv(filename)
			elif filetype == 'txt':
				data = pd.read_csv(filename, delimiter=dlm)
		return name, data
	except:
		if not os.path.isdir(filename):
			logging.info(f'Unable to read {os.path.basename(filename)}')
	return name, data

def get_testdir():
	fcdir = os.path.dirname(os.path.realpath(__file__))
	datapath = os.path.join(fcdir, 'testdata')
	return datapath

def buildfilename(cellid, test, num=None, filetype='csv'):
	if num:
		filename = cellid + '_' + test + str(num) + '.' + filetype
	else:
		filename = cellid + '_' + test + '.' + filetype
	return filename

def testfun():
	return ', '.join(valid_types)